@@ -17,7 +17,10 @@ from tarantool.const import (
     IPROTO_ERROR,
     IPROTO_SYNC,
     IPROTO_SCHEMA_ID,
-    REQUEST_TYPE_ERROR
+    REQUEST_TYPE_ERROR,
+    IPROTO_SQL_INFO,
+    IPROTO_SQL_INFO_ROW_COUNT,
+    IPROTO_SQL_INFO_AUTOINCREMENT_IDS
 )
 from tarantool.error import (
     DatabaseError,
@@ -50,15 +53,39 @@ class Response(Sequence):
         # created in the __new__().
         # super(Response, self).__init__()
 
+        unpacker_kwargs = dict()
+
+        # Decode msgpack arrays into Python lists by default (not tuples).
+        # Can be configured in the Connection init
+        unpacker_kwargs['use_list'] = conn.use_list
+
+        # Use raw=False instead of encoding='utf-8'.
         if msgpack.version >= (0, 5, 2) and conn.encoding == 'utf-8':
             # Get rid of the following warning.
             # > PendingDeprecationWarning: encoding is deprecated,
             # > Use raw=False instead.
-            unpacker = msgpack.Unpacker(use_list=True, raw=False)
+            unpacker_kwargs['raw'] = False
         elif conn.encoding is not None:
-            unpacker = msgpack.Unpacker(use_list=True, encoding=conn.encoding)
-        else:
-            unpacker = msgpack.Unpacker(use_list=True)
+            unpacker_kwargs['encoding'] = conn.encoding
+
+        # raw=False is default since msgpack-1.0.0.
+        #
+        # The option decodes mp_str to bytes, not a Unicode
+        # string (when True).
+        if msgpack.version >= (1, 0, 0) and conn.encoding is None:
+            unpacker_kwargs['raw'] = True
+
+        # encoding option is not supported since msgpack-1.0.0,
+        # but it is handled in the Connection constructor.
+        assert(msgpack.version < (1, 0, 0) or conn.encoding in (None, 'utf-8'))
+
+        # strict_map_key=True is default since msgpack-1.0.0.
+        #
+        # The option forbids non-string keys in a map (when True).
+        if msgpack.version >= (1, 0, 0):
+            unpacker_kwargs['strict_map_key'] = False
+
+        unpacker = msgpack.Unpacker(**unpacker_kwargs)
 
         unpacker.feed(response)
         header = unpacker.unpack()
@@ -236,7 +263,7 @@ class Response(Sequence):
                     'code': self.strerror[0],
                     'reason': self.return_message
                 }
-            }, sort_keys = True, indent = 4)
+            }, sort_keys = True, indent = 4, separators=(', ', ': '))
         output = []
         for tpl in self._data or ():
             output.extend(("- ", repr(tpl), "\n"))
@@ -245,3 +272,43 @@ class Response(Sequence):
         return ''.join(output)
 
     __repr__ = __str__
+
+
+class ResponseExecute(Response):
+    @property
+    def autoincrement_ids(self):
+        """
+        Returns a list with the new primary-key value
+        (or values) for an INSERT in a table defined with
+        PRIMARY KEY AUTOINCREMENT
+        (NOT result set size)
+
+        :rtype: list or None
+        """
+        if self._return_code != 0:
+            return None
+        info = self._body.get(IPROTO_SQL_INFO)
+
+        if info is None:
+            return None
+
+        autoincrement_ids = info.get(IPROTO_SQL_INFO_AUTOINCREMENT_IDS)
+
+        return autoincrement_ids
+
+    @property
+    def affected_row_count(self):
+        """
+        Returns the number of changed rows for responses
+        to DML requests and None for DQL requests.
+
+        :rtype: int
+        """
+        if self._return_code != 0:
+            return None
+        info = self._body.get(IPROTO_SQL_INFO)
+
+        if info is None:
+            return None
+
+        return info.get(IPROTO_SQL_INFO_ROW_COUNT)
