@@ -1,27 +1,34 @@
 # -*- coding: utf-8 -*-
 import sys
-import base64
 import uuid
 
 # Compatibility layer for Python2/Python3
 if sys.version_info.major == 2:
     string_types     = (basestring, )
     integer_types    = (int, long)
+    supported_types  = integer_types + string_types + (float,)
+
     ENCODING_DEFAULT = None
+
     if sys.version_info.minor < 6:
         binary_types = (str, )
     else:
         binary_types = (bytes, )
+    from base64 import decodestring as base64_decode
 
     def strxor(rhs, lhs):
         return "".join(chr(ord(x) ^ ord(y)) for x, y in zip(rhs, lhs))
 
 elif sys.version_info.major == 3:
-    binary_types  = (bytes, )
-    string_types  = (str, )
-    integer_types = (int, )
+    binary_types    = (bytes, )
+    string_types    = (str, )
+    integer_types   = (int, )
+    supported_types = integer_types + string_types + binary_types + (float,)
+
     ENCODING_DEFAULT = "utf-8"
 
+    from base64 import decodebytes as base64_decode
+
     def strxor(rhs, lhs):
         return bytes([x ^ y for x, y in zip(rhs, lhs)])
 
@@ -42,7 +49,7 @@ def check_key(*args, **kwargs):
         elif args[0] is None and kwargs['select']:
             return []
     for key in args:
-        assert isinstance(key, integer_types + string_types + (float,))
+        assert isinstance(key, supported_types)
     return list(args)
 
 
@@ -62,7 +69,7 @@ def greeting_decode(greeting_buf):
     # Tarantool 1.6.8-132-g82f5424 (Lua console)
     result = Greeting()
     try:
-        (product, _, tail) = str(greeting_buf)[0:63].partition(' ')
+        (product, _, tail) = greeting_buf[0:63].decode().partition(' ')
         if product.startswith("Tarantool "):
             raise Exception()
         # Parse a version string - 1.6.6-83-gc6b2129 or 1.6.7
@@ -84,8 +91,8 @@ def greeting_decode(greeting_buf):
             # Tarantool < 1.6.7 doesn't add "(Binary)" to greeting
             result.protocol = "Binary"
         elif len(tail.strip()) != 0:
-            raise Exception("x")  # Unsuported greeting
-        result.salt = base64.decodestring(greeting_buf[64:])[:20]
+            raise Exception("x")  # Unsupported greeting
+        result.salt = base64_decode(greeting_buf[64:])[:20]
         return result
     except Exception as e:
         print('exx', e)
